/* Copyright 2020, Michele Santullo
 * This file is part of orotool.
 *
 * Orotool is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Orotool is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Orotool.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "rest_server.hpp"
#include "CivetServer.h"
#include "duckhandy/int_conv.hpp"
#include "private/dateconv.hpp"

namespace duck {
namespace {
class V1ItemsHandler : public CivetHandler {
public:
	bool handleGet (CivetServer* server, struct mg_connection* conn) override {
		mg_printf(conn,
			"HTTP/1.1 200 OK\r\n"
			"Content-Type: application/json; charset=utf-8\r\n"
			"Connection: close\r\n"
			"Date: %s\r\n"
			"\r\n",
			to_header_string(oro::Timestamp::now()).c_str()
		);
		mg_printf(conn, "Hi this is the items handler\r\n");
		return true;
	}
};

void init_civetweb_ifn() {
	static bool initialised = false;
	if (not initialised) {
		initialised = true;
		if (mg_check_feature(2))
			mg_init_library(MG_FEATURES_SSL);
		else
			mg_init_library(0);
	}
}
} //unnamed namespace

struct RestServer::LocalData {
	explicit LocalData (const char* options[]) :
		server(options)
	{
		server.addHandler("/v1/items", v1_items_handler);
	}

	V1ItemsHandler v1_items_handler;
	CivetServer server;
};

RestServer::RestServer (uint16_t port) {
	init_civetweb_ifn();

	const std::string port_str = dhandy::int_conv<std::string>(port);
	const char* options[] = {
		"decode_url", "yes",
		"listening_ports", port_str.c_str(),
		"request_timeout_ms", "10000",
		nullptr
	};

	m_local = std::make_unique<LocalData>(options);
}

RestServer::~RestServer() noexcept = default;
} //namespace duck
